<?php

namespace Drupal\drupalorg_crosssite\EventSubscriber;

use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Site\Settings;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Event handler that restricts login to only certain users.
 */
class AllowedLogin implements EventSubscriberInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  private $currentUser;

  /**
   * AllowedLogin constructor.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   */
  public function __construct(AccountInterface $current_user) {
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    // The priority for this must run directly after the authentication
    // subscriber.
    $events[KernelEvents::REQUEST][] = ['checkUser', 300];
    return $events;
  }

  /**
   * Checks the user allowed to log in.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The Event to process.
   */
  public function checkUser(RequestEvent $event) {
    $allowed_users = Settings::get('drupalorg_crosssite_allowed_users');
    if (is_array($allowed_users) && array_search('*', $allowed_users) === FALSE) {
      if (
        $this->currentUser->isAuthenticated() &&
        !in_array(strtolower($this->currentUser->getAccountName()), $allowed_users)
      ) {
        // Log the user out and redirect to the www site.
        user_logout();
        $redirect_url = ($event->getRequest()->getRequestUri() === '/user') ?
          'https://www.drupal.org/user' :
          'https://www.drupal.org';
        $response = new TrustedRedirectResponse($redirect_url);
        $event->setResponse($response);
      }
    }

    return $event;
  }

}
